/*******************************************************************
*
*  DESCRIPTION: class Contador (implementation)
*
*  AUTHOR: 
*
*  DATE: 
*
*******************************************************************/

/** include files **/
#include "tsccounter.h"   // base class
#include "message.h"    // class ExternalMessage 
#include "mainsimu.h"       // class MainSimulator

/*******************************************************************
* Function Name: TSCCounter
* Description: Constructor
********************************************************************/
TSCCounter::TSCCounter( const string &name )
: Atomic( name ) 
, arrived( addInputPort( "arrived" ) )
, solved( addInputPort( "solved" ) )
, throughputA( addOutputPort( "throughputA" ) )
, throughputS( addOutputPort( "throughputS" ) )
{
	frec = "0:1:0:0" ;
	tUnit = "0:1:0:0" ;

	if( MainSimulator::Instance().existsParameter( description(), "frecuence" ) )
		frec = MainSimulator::Instance().getParameter( description(), "frecuence" ) ;

	if( MainSimulator::Instance().existsParameter( description(), "timeUnit" ) )
		tUnit = MainSimulator::Instance().getParameter( description(), "timeUnit" ) ;
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCCounter::initFunction()
{
	arrivedCount = 0 ;
	solvedCount = 0;
	holdIn( active, frecuence() ) ;
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCCounter::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == arrived )
	{  
		arrivedCount ++;		
	}

	if( msg.port() == solved )
	{
		
		solvedCount ++ ;
		
	}
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCCounter::internalFunction( const InternalMessage &)
{
	holdIn( active, frecuence() );
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCCounter::outputFunction( const InternalMessage &msg )
{
	float time( msg.time().asMsecs() / timeUnit().asMsecs() ) ;

	sendOutput( msg.time(), throughputA, arrivedCount / time ) ;
	sendOutput( msg.time(), throughputS, solvedCount / time ) ;

	
	return *this ;
}
