/** include files **/
#include "tscsincrotrafficlight.h"      // base header
#include "message.h"           // class InternalMessage 
#include "parsimu.h"          // class Simulator
#include "distri.h"            // class Distribution 
#include "strutil.h"           // str2Value( ... )
#include "stdlib.h"
#include "atomicstate.h"

/*******************************************************************
* Function Name: TSCSincroTrafficLight
* Description: constructor
********************************************************************/
TSCSincroTrafficLight::TSCSincroTrafficLight( const string &name )
: Atomic( name )

{
	try
	{
		dist = Distribution::create( ParallelMainSimulator::Instance().getParameter( description(), "distribution" ) );
		MASSERT( dist ) ;
		for ( register int i = 0; i < dist->varCount(); i++ )
		{
			string parameter( ParallelMainSimulator::Instance().getParameter( description(), dist->getVar( i ) ) ) ;
			dist->setVar( i, str2Value( parameter ) ) ;
		}

		if( ParallelMainSimulator::Instance().existsParameter( description(), "tlights" ) )
			tlights = str2Int( ParallelMainSimulator::Instance().getParameter( description(), "tlights" ) );
            else 
			tlights = 0;

		if (tlights == 0) {
                MTHROW(MException("TSCSincroTrafficLight: model " + description() + " has tlights = 0 !" ));
                }

	} catch( InvalidDistribution &e )
	{
		e.addText( "The model " + description() + " has distribution problems!" ) ;
		e.print(cerr);
		MTHROW( e ) ;
	} catch( MException &e )
	{
		MTHROW( e ) ;
	}

	int i;
	string s;
	char buff[5];
	for (i=0; i <tlights; i++){
		//s = "y_se_luz";
		//s += itoa(i,buff,10);
		sprintf(buff, "y_se_luz%d", i);
                                  s = buff;
		y_se_luz.insert(map<int,Port*>::value_type(i,&addOutputPort(s)));
	}

}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCSincroTrafficLight::initFunction()
{
      tlight = 0;
	holdIn( AtomicState::active, VTime::Zero ) ;
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCSincroTrafficLight::internalFunction( const InternalMessage & )
{
	holdIn( AtomicState::active, VTime( static_cast< float >( fabs( distribution().get() ) ) ) ) ;
      tlight = tlight + 1;
      if (tlight == tlights)
         tlight = 0;
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCSincroTrafficLight::outputFunction( const InternalMessage &msg )
{
  int i;
 
  for (i=0; i <tlights; i++){
      if (i==tlight)
   		sendOutput( msg.time(), *(y_se_luz[i]), 0 );  // green light
	else
   		sendOutput( msg.time(), *(y_se_luz[i]), 1 );  // red light
  }
  return *this ;
}


/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCSincroTrafficLight::externalFunction( const ExternalMessage &msg ){
	throw InvalidMessageException();
}


TSCSincroTrafficLight::~TSCSincroTrafficLight()
{
	delete dist;
      delete &y_se_luz;
}
