#include "segment.h"
#include <math.h>

void      Segment::setIdent(string id)      {this->ident=id;}
void      Segment::setFirstPoint(Point p)   {this->firstPoint=p;}
void      Segment::setSecondPoint(Point p)  {this->secondPoint=p;}

void      Segment::setQlane(int ql){
if (ql==0){
    string m = "Quantity of lanes equal 0 for segment ";
    m += this->ident;
    throw TException("setQlane",m);
    }    
this->qlane=ql;
}

void      Segment::setDirec(string d)        {this->direc=d;}
void      Segment::setForm(string f)         {this->form=f;}
void      Segment::setSpeed(int sp)          {this->speed=sp;}

void      Segment::setQcell(){
if (this->form==""){  
    string m = "Invalid data for segment ";
    m += this->ident;
    throw TException("setQcell",m);
    }    
const float pi = 3.1415926;
double length = sqrt(pow(this->firstPoint.getx() - this->secondPoint.getx(),2) 
                 + pow(this->firstPoint.gety() - this->secondPoint.gety(),2));

if (this->form=="straight")
   this->qcell = (int) length;
else
   this->qcell = (int)(pi * length /2);
}


void      Segment::setDelay(int d){delay = d;}
int       Segment::getDelay(){return delay;}
string    Segment::getIdent()               {return this->ident;}
Point     Segment::getFirstPoint()          {return this->firstPoint;}
Point     Segment::getSecondPoint()         {return this->secondPoint;}

void      Segment::setParkType(string pt)    {
if (pt=="parkBoth" && this->qlane < 4){
    string m = "parkBoth in segment ";
    m += this->ident;
    m += " with less than 4 lanes ";
    throw TException("setParkType",m);
    }        
else
   if ((pt=="parkLeft" ||pt=="parkRight")&& this->qlane < 2){
       string m = "parking in segment ";
       m += this->ident;
       m += " with less than 2 lanes ";
       throw TException("setParkType",m);
       }        
   else this->parkType=pt;

int i;
Hole *h;

if (pt=="parkBoth" || pt=="parkLeft")
   for(i=0;i<this->qcell;i++){
      h = new Hole;
      h->setLane(1);
      h->setDistance(i);
      h->setDelay(this->delay);
      h->setSegment(this);
      this->addHole(h);
   }

if (pt=="parkBoth" || pt=="parkRight")
   for(i=0;i<this->qcell;i++){
      h = new Hole;
      h->setLane(this->qlane);
      h->setDistance(i);
      h->setDelay(this->delay);
      h->setSegment(this);
      this->addHole(h);
   }

}


string Segment::getParkType()            {return this->parkType;}

string Segment::getTLight() {
Crossing * endCross = this->getEndCross();
if(endCross!= NULL && endCross->isInSeg(this))
   return endCross->getTLight();
else
   return "withoutTL";
}

Point     Segment::getStartPoint() {      
   if (this->direc=="go")
      return this->getFirstPoint();
   else
     return this->getSecondPoint();
}

Point     Segment::getEndPoint() {
   if (this->direc=="go")
      return this->getSecondPoint();
   else
     return this->getFirstPoint();
}

int       Segment::getQlane()            {return this->qlane;}
string    Segment::getDirec()            {return this->direc;}
string    Segment::getForm()             {return this->form;}
int       Segment::getSpeed()            {return this->speed;}
int       Segment::getQcell()            {return this->qcell;}
double    Segment::getFirstIncl()        {return this->firstIncl;}
double    Segment::getSecondIncl()       {return this->secondIncl;}

double    Segment::getStartIncl(){
   if (this->direc=="go")
      return this->firstIncl;
   else
     return this->secondIncl;
}

double    Segment::getEndIncl(){
   if (this->direc=="go")
      return this->secondIncl;
   else
     return this->firstIncl;
}

Crossing *Segment::getFirstCross()                 {return this->firstCross;}
Crossing *Segment::getSecondCross()                {return this->secondCross;}
void      Segment::setFirstCross(Crossing *c)      {this->firstCross=c;}
void      Segment::setSecondCross(Crossing *c )    {this->secondCross=c;}

Crossing *Segment::getStartCross(){
   if (this->direc=="go")
      return this->firstCross;
   else
     return this->secondCross;
}

Crossing *Segment::getEndCross(){
   if (this->direc=="go")
      return this->secondCross;
   else
     return this->firstCross;
}


Point Segment::getMinorPoint(){
   if (this->getFirstPoint() < this->getSecondPoint())
	   return this->getFirstPoint();
   else
	   return this->getSecondPoint();
}

Point Segment::getMajorPoint(){
   if (this->getFirstPoint() > this->getSecondPoint())
	   return this->getFirstPoint();
   else
	   return this->getSecondPoint();
}

double Segment::getMinorIncl(){
   if (this->firstIncl < this->secondIncl)
	   return this->firstIncl;
   else
	   return this->secondIncl;
}

void Segment::calcIncls(){
double incl;
const float pi = 3.1415926;

if (this->getFirstPoint().getx() == this->getSecondPoint().getx()) 
// hay que separa este caso pues hay diferencias si se toma el mismo segmento al revs
   incl = pi/2;   
else
   incl = asin ( abs(this->getFirstPoint().gety() - this->getSecondPoint().gety())/
                 sqrt(pow(this->getFirstPoint().gety() - this->getSecondPoint().gety(), 2) +
                      pow(this->getFirstPoint().getx() - this->getSecondPoint().getx(), 2)));
if (this->getFirstPoint() < this->getSecondPoint()) 
   if (this->getFirstPoint().getx() <= this->getSecondPoint().getx()) {
       this->firstIncl = incl;
       this->secondIncl = incl + pi;}
   else{
       this->firstIncl = pi - incl;
       this->secondIncl = 2 * pi - incl;}
else 
   if (this->getFirstPoint().getx() <= this->getSecondPoint().getx()) {
       this->firstIncl = 2 * pi - incl;
       this->secondIncl = pi - incl;}
   else{
       this->firstIncl = incl + pi;
       this->secondIncl = incl;}
}


void Segment::print(){
   cout << "Segment " << this->ident << "|";
   this->getFirstPoint().print();
   cout << "|";
   this->getSecondPoint().print();
   cout << "|";
   cout << this->qlane               << "|"
        << this->form                << "|"
        << this->direc               << "|"
        << this->speed               << "|"
        << this->qcell               << "|"
        << this->parkType            << "|"
        << this->delay               << "|"
        << this->firstIncl           << "|"
        << this->secondIncl          << "|";

   if(this->getStartCross()!= NULL) 
       cout << "\n   Start Cross(" << this->getStartCross()->getIdent()<<")";

   if(this->getEndCross()!= NULL) 
       cout << "\n   End Cross(" << this->getEndCross()->getIdent()<<")";
   
   list<Jobsite>::iterator   iterJ;
   if (this->lJobsite.size() != 0)
      for (iterJ = this->lJobsite.begin(); iterJ != this->lJobsite.end(); iterJ++)
          {cout << "\n   ";iterJ->print();};

   list<Hole>::iterator   iterH;
   if (this->lHole.size() != 0)
      for (iterH = this->lHole.begin(); iterH != this->lHole.end(); iterH++)
          {cout << "\n   ";iterH->print();};

   list<CtrElem>::iterator   iterC;
   if (this->lCtrElem.size() != 0)
      for (iterC = this->lCtrElem.begin(); iterC != this->lCtrElem.end(); iterC++)
          {cout << "\n   ";iterC->print();};

   list<Railnet*>::iterator   iterR;
   if (this->lRailnet.size() != 0)
      for (iterR = this->lRailnet.begin(); iterR != this->lRailnet.end(); iterR++)
          {cout << "\n   Railnet(";
           cout << (*iterR)->getIdent() << ",";
           cout << (*iterR)->findDistance(this->ident);
           cout << ")";
          };
}



void Segment::addJobsite(Jobsite* j) {
if (j->getLane()== 0 || j->getLane() > this->qlane ) {
    string m = "Jobsite first lane is 0 or out of segment ";
    m += this->ident;
    throw TException("addJobsite",m);
    }    
else if (j->getDistance() >= this->qcell ) {
        string m = "Jobsite distance out of segment ";
        m += this->ident;
        throw TException("addJobsite",m);
        }    
     else if (j->getDistance()==0 || j->getDistance()== this->qcell -1){
             string m = "Jobsite distance in border of segment ";
             m += this->ident;
             throw TException("addJobsite",m);
             }        
          else if (j->getQlane()==0 || j->getQlane() > this->qlane ) {
                  string m = "Jobsite last lane is 0 out of segment ";
                  m += this->ident;
                  throw TException("addJobsite",m);
                  }        
               else {int quot = j->getQlane()/2;
                     if (j->getQlane()== quot *2) {
                        string m = "Jobsite with odd number of lanes for segment ";
                        m += this->ident;
                        throw TException("addJobsite",m);
                        }
                     else this->lJobsite.push_back(*j);
               }
}



void Segment::addHole(Hole* h)       {
if (h->getLane()==0 || h->getLane() > this->qlane ) {   
    string m = "Hole lane is 0 or out of segment ";
    m += this->ident;
    throw TException("addHole",m);
    }
else
   if (h->getDistance() >= this->qcell ) {   
       string m = "Hole distance out of segment ";
       m += this->ident;
       throw TException("addHole",m);
       }
   else this->lHole.push_back(*h);
}



void Segment::addCtrElem(CtrElem* c) { 
if (c->getDistance() >= this->qcell ) {
    string m = "Control element distance out of segment ";
    m += this->ident;
    throw TException("addCtrElem",m);
    }
else this->lCtrElem.push_back(*c);
}



void Segment::addRailnet(Railnet* r) {
int d = r->findDistance(this->ident);
if ( d >= this->qcell ) {
    string m = "Railnet ";
    m += r->getIdent();
    m += ", distance out of segment ";
    m += this->ident;
    throw TException("addRailnet",m);
    }    
else if (d==0 || d==this->qcell -2) { //tengo que considerar before y after
        string m = "Railnet ";
        m += r->getIdent();
        m += ", distance in border of segment ";
        m += this->ident;
        throw TException("addRailnet",m);
        }
else if (this->findRailnet(r->getIdent())!=NULL) {
        string m = "Railnet ";
        m += r->getIdent();
        m += ", more than one time in segment ";
        m += this->ident;
        throw TException("addRailnet",m);
        }
     else this->lRailnet.push_back(r);
} 


list<Railnet*> * Segment::getLRailnet() {return &(this->lRailnet);}
list<Jobsite>  * Segment::getLJobsite() {return &(this->lJobsite);}
list<Hole>     * Segment::getLHole()    {return &(this->lHole);}
list<CtrElem>  * Segment::getLCtrElem() {return &(this->lCtrElem);}


Railnet* Segment::findRailnet(string r){
bool ubic= false;
list< Railnet*>::iterator   iter;
iter = this->lRailnet.begin();

while ( iter != this->lRailnet.end() && !ubic )
   if ((*iter)->getIdent()==r) ubic = true;
   else iter++;

if(ubic == true) return (*iter);
else return NULL; 
};
