%{
#include <stdio.h>
#include <iostream>
#define YYDEBUG 1
#define YYERROR_VERBOSE 1
#include "traffic.h"
#include "point.h"
#include "segment.h"
#include "crossing.h"
#include "texception.h"
int yyerror(char *s);
int yylex(void);

extern int errorline;

extern Traffic traffic;
Segment   *s;
Point     *p;
Crossing  *c;
Hole      *h;
Jobsite   *j;
CtrElem   *ce;
Railnet   *r;
bool      hasSegs=false;
%}

%union {
        int   i;
        char *s;
       }

%type <s> PROGRAM LIST ELEMENT
%type <s> SEGMENTS  LIST_SEGMENTS  SEGMENT
%type <s> CROSSINGS LIST_CROSSINGS CROSSING 
%type <s> POINT DIRECTION SHAPE TLIGHT PARKTYPE CROSSHOLE
%type <s> RAILNETS LIST_RAILNETS RAILNET LIST_RAILSEGMENTS RAILSEGMENT
%type <s> JOBSITES LIST_JOBSITES JOBSITE
%type <s> HOLES LIST_HOLES HOLE
%type <s> CTRELEMS LIST_CTRELEMS CTRELEM CTRTYPE
%type <i> LANES SPEED DELAY DISTANCE POUT

%token <s> TOK_IDENTIF         "identifier"
%token <s> TOK_NUMBER          "number" 
%token <s> TOK_BEGIN           "begin"
%token <s> TOK_END             "end"
%token <s> TOK_SEGMENTS        "segments"
%token <s> TOK_CROSSINGS       "crossings"
%token <s> TOK_GO              "go"
%token <s> TOK_BACK            "back"
%token <s> TOK_STRAIGHT        "straight"
%token <s> TOK_CURVE           "curve"
%token <s> TOK_WITH_TLIGHT     "withTL"
%token <s> TOK_WITHOUT_TLIGHT  "withoutTL"
%token <s> TOK_WITH_HOLE       "withHole"
%token <s> TOK_WITHOUT_HOLE    "withoutHole"
%token <s> TOK_RAILNETS        "railnets"
%token <s> TOK_IN              "in"
%token <s> TOK_JOBSITES        "jobsites"
%token <s> TOK_HOLES           "holes"
%token <s> TOK_CTRELEMS        "ctrElements"
%token <s> TOK_CTRTYPE1        "sawhorse"
%token <s> TOK_CTRTYPE2        "depression"
%token <s> TOK_CTRTYPE3        "intersection"
%token <s> TOK_CTRTYPE4        "saw"
%token <s> TOK_CTRTYPE5        "stop"
%token <s> TOK_CTRTYPE6        "school"
%token <s> TOK_PARKNONE        "parkNone"
%token <s> TOK_PARKLEFT        "parkLeft"
%token <s> TOK_PARKRIGHT       "parkRight"
%token <s> TOK_PARKBOTH        "parkBoth"



%token <s> '(' ')' ',' '=' 
%token <s> TOK_ERROR           
%%

PROGRAM            :                   {yyerror("invalid program");}
                   |LIST               {if (!hasSegs) yyerror("invalid program");}
                   ;

LIST               :ELEMENT             
                   |LIST ELEMENT        
                   ;

ELEMENT            :SEGMENTS              {hasSegs=true;}
                   |CROSSINGS             
                   |RAILNETS              
                   |JOBSITES              
                   |HOLES                 
                   |CTRELEMS              
                   ;


SEGMENTS           :TOK_BEGIN TOK_SEGMENTS LIST_SEGMENTS TOK_END TOK_SEGMENTS
                   ;

LIST_SEGMENTS      :LIST_SEGMENTS SEGMENT 
                   |SEGMENT
                   ;

SEGMENT            :TOK_IDENTIF           {s = new Segment;
                                           s->setIdent(strdup($1));
                                           p = new Point;} 
                   '='                         
                   POINT 		      {s->setFirstPoint(*p);
                                           p = new Point;}
                   ',' POINT 		      {s->setSecondPoint(*p);} 
                   ',' LANES              {s->setQlane($10);}   
                   ',' SHAPE              {s->setForm(strdup($13));
                                           s->setQcell();}
                   ',' DIRECTION          {s->setDirec(strdup($16));}
                   ',' SPEED              {s->setSpeed($19);}
                   ',' DELAY              {s->setDelay($22);}
                   ',' PARKTYPE           {s->setParkType(strdup($25));
                                           traffic.addSeg(s);}
                   ; 


CROSSINGS          :TOK_BEGIN TOK_CROSSINGS LIST_CROSSINGS TOK_END TOK_CROSSINGS  
                   ;

LIST_CROSSINGS     :LIST_CROSSINGS CROSSING
                   |CROSSING  
                   ;

CROSSING           :TOK_IDENTIF           {c = new Crossing;
                                           c->setIdent(strdup($1));
                                           p = new Point;} 
                   '=' POINT  		{c->setPoint(*p);}
                   ',' SPEED              {c->setSpeed($7);}
                   ',' TLIGHT             {c->setTLight(strdup($10));}
                   ',' CROSSHOLE          {c->setHole(strdup($13));}
                   ',' DELAY              {c->setDelay($16);}
                   ',' POUT               {c->setPout($19);
                                           traffic.addCross(c);}
                   ;


RAILNETS           :TOK_BEGIN TOK_RAILNETS LIST_RAILNETS TOK_END TOK_RAILNETS
                   ;

LIST_RAILNETS      :LIST_RAILNETS RAILNET 
                   |RAILNET
                   ;

RAILNET            :TOK_IDENTIF           {r = new Railnet;
                                           r->setIdent(strdup($1));}
                   '=' LIST_RAILSEGMENTS  
                   ',' DELAY              {r->setDelay($6);
                                           traffic.addRailnet(r);}
                   ;

LIST_RAILSEGMENTS  :LIST_RAILSEGMENTS ',' RAILSEGMENT 
                   |RAILSEGMENT 
                   ;

RAILSEGMENT        :'(' TOK_IDENTIF       {s=traffic.findSeg(strdup($2));
                                           if (s==NULL) yyerror("defineRailNet");}
                   ',' TOK_NUMBER         {r->addRail(s,atoi($5));
                                           s->addRailnet(r);}
                   ')'
                   ;


JOBSITES           :TOK_BEGIN TOK_JOBSITES LIST_JOBSITES TOK_END TOK_JOBSITES
                   ;

LIST_JOBSITES      :LIST_JOBSITES JOBSITE
                   |JOBSITE
                   ;

JOBSITE            :TOK_IN
                    TOK_IDENTIF           {s=traffic.findSeg(strdup($2));
                                           if (s==NULL) yyerror("defineJobSite");
                                           j= new Jobsite;}
                   ':' LANES              {j->setLane($5);}
                   ',' DISTANCE           {j->setDistance($8);} 
                   ',' LANES              {j->setQlane($11);}
                   ',' DELAY              {j->setDelay($14);
                                           j->setSegment(s);
                                           s->addJobsite(j);}
                   ;


HOLES              :TOK_BEGIN TOK_HOLES LIST_HOLES TOK_END TOK_HOLES
                   ;

LIST_HOLES         :LIST_HOLES HOLE
                   |HOLE
                   ;

HOLE               :TOK_IN
                    TOK_IDENTIF           {s=traffic.findSeg(strdup($2));
                                           if (s==NULL) yyerror("defineHole");
                                           h= new Hole;}
                   ':' LANES              {h->setLane($5);} 
                   ',' DISTANCE           {h->setDistance($8);}
                   ',' DELAY              {h->setDelay($11);
                                           h->setSegment(s);
                                           s->addHole(h);}
                   ;


CTRELEMS           :TOK_BEGIN TOK_CTRELEMS LIST_CTRELEMS TOK_END TOK_CTRELEMS
                   ;

LIST_CTRELEMS      :LIST_CTRELEMS CTRELEM
                   |CTRELEM
                   ;

CTRELEM            :TOK_IN
                    TOK_IDENTIF           {s=traffic.findSeg(strdup($2));
                                           if (s==NULL) yyerror("defineCtrElement");
                                           ce= new CtrElem;}
                   ':' CTRTYPE            {ce->setType(strdup($5));}
                   ',' DISTANCE           {ce->setDistance($8);}
                   ',' DELAY              {ce->setDelay($11);
                                           ce->setSegment(s);
                                           s->addCtrElem(ce);}
                   ;

POINT              :'(' TOK_NUMBER        {p->setx(atoi($2));}
                   ',' TOK_NUMBER         {p->sety(atoi($5));} 
                   ')'
                   ;

DIRECTION          :TOK_GO
                   |TOK_BACK
                   ;

SHAPE              :TOK_CURVE
                   |TOK_STRAIGHT                 
                   ;

TLIGHT             :TOK_WITH_TLIGHT
                   |TOK_WITHOUT_TLIGHT
                   ;

PARKTYPE           :TOK_PARKNONE
                   |TOK_PARKLEFT
                   |TOK_PARKRIGHT
                   |TOK_PARKBOTH 
                   ;

CROSSHOLE          :TOK_WITH_HOLE
                   |TOK_WITHOUT_HOLE
                   ;

CTRTYPE            :TOK_CTRTYPE1
                   |TOK_CTRTYPE2
                   |TOK_CTRTYPE3
                   |TOK_CTRTYPE4
                   |TOK_CTRTYPE5
                   |TOK_CTRTYPE6
                   ;

LANES              :TOK_NUMBER            {$$=atoi($1);}
                   ;

SPEED              :TOK_NUMBER            {$$=atoi($1);}
                   ;

DELAY              :TOK_NUMBER            {$$=atoi($1);}
                   ;

DISTANCE           :TOK_NUMBER            {$$=atoi($1);}
                   ;

POUT               :TOK_NUMBER            {$$=atoi($1);}
                   ;

%%


int yyerror(char *s){
/*********
#define YYERROR_VERBOSE 1                   para que el error sea mas detallado
%token <s> TOK_IDENTIF  "identifier"        "identifier" es lo que escribe si es lo que est esperando
*******/
char m[512];

if (strncmp(s,"parse error",11)==0)
    sprintf(m,"line #: %d --> %s not expected",errorline, yylval.s);
else if (strcmp(s,"invalid program")==0)
     	strcpy(m,"The program must start with begin and must have segments");
     else sprintf(m,"line #: %d --> %s invalid segment",errorline, yylval.s);

throw TException(s,strdup(m));
}



