#ifndef __SEGMENT_H
#define __SEGMENT_H

#include <list>
#include <istream>
#include <string>
#include "texception.h"
#include "point.h"
#include "crossing.h"
#include "jobsite.h"
#include "ctrelem.h"
#include "hole.h"
#include "railnet.h"

//forward declarations
class TException;
class Point;
class Crossing;
class Jobsite;
class CtrElem;
class Hole;
class Railnet;

class Segment {
public:
    void      setIdent(std::string);
    void      setFirstPoint(Point);
    void      setSecondPoint(Point);
    void      setQlane(int);
    void      setDirec(std::string);
    void      setForm(std::string);
    void      setSpeed(int);
    void      setQcell();
    void      setDelay(int);
    void      setParkType(std::string);
    std::string    getIdent();
    Point     getFirstPoint();
    Point     getSecondPoint();
    Point     getStartPoint();
    Point     getEndPoint();
    int       getQlane();
    std::string    getDirec();
    std::string    getForm();
    int       getSpeed();
    int       getQcell();
    int       getDelay();
    std::string    getParkType();
    Point     getMinorPoint();
    Point     getMajorPoint();
    double    getFirstIncl();
    double    getSecondIncl();
    double    getStartIncl();
    double    getEndIncl();
    double    getMinorIncl();
    void      calcIncls();
    Crossing* getFirstCross();
    Crossing* getSecondCross();
    Crossing* getStartCross();
    Crossing* getEndCross();
    void      setFirstCross(Crossing*);
    void      setSecondCross(Crossing*);
    std::string    getTLight();
    void      print();                 //print the segment info to the stdin


    void           addRailnet(Railnet*); 
    std::list<Railnet*> *getLRailnet();
    Railnet*       findRailnet(std::string);

    void           addJobsite(Jobsite*); 
    std::list<Jobsite>  *getLJobsite();

    void           addHole(Hole*);     
    std::list<Hole>    *getLHole();

    void            addCtrElem(CtrElem*); 
    std::list<CtrElem>   *getLCtrElem();

protected:
    std::string    ident;
    Point     firstPoint;
    Point     secondPoint;
    int       qlane;
    std::string    direc;
    std::string    form;
    int       speed;
    int       qcell;
    int       delay;
    std::string    parkType;
    double    firstIncl;
    double    secondIncl;
    Crossing *firstCross;
    Crossing *secondCross;
    std::list<Jobsite>  lJobsite;
    std::list<Hole>     lHole;
    std::list<CtrElem>  lCtrElem;
    std::list<Railnet*> lRailnet;

private:
};


#endif /* __SEGMENT_H */
